#include <stdio.h>
#include <stdlib.h>
#include "bstack.h"

/** Public Functions */

/* Allocates resources for the stack */

BoundedStack * BSCreate(unsigned int size)
{
  int i = 0;
  BoundedStack * stack = (BoundedStack *) malloc(sizeof(BoundedStack));
  BSNode * thisnode = NULL;

  stack->top = 0;
  stack->size = size;
  stack->used = 0;
  stack->nodes = (BSNode *) malloc(sizeof(BSNode) * size);

  thisnode = stack->nodes;

  for (i = 0; i < size; i++)
  {
    thisnode->data = NULL;
    thisnode++;
  }

  return stack;
}

/* Pushes a value onto the stack */

void BSPush(BoundedStack * stack, void * data)
{
  /* stack is full */
  if (stack->used == stack->size)
  {
    BSNode * overwritenode = NULL;
    stack->top = (stack->top + 1) % stack->size;
    overwritenode = stack->nodes + (stack->top * sizeof(BSNode));
    overwritenode->data = data; /* overwrite old bottom (which is now top) */
  }
  /* stack is not full */
  else
  {
    BSNode * newnode = NULL;
    if (stack->used != 0)
      stack->top = (stack->top + 1) % stack->size;
    newnode = stack->nodes + (stack->top * sizeof(BSNode));
    newnode->data = data;
    stack->used++;
  }
}

void * BSPop(BoundedStack * stack)
{
  /* stack is empty */
  if (stack->used == 0)
    return NULL;
  /* stack is not empty */
  else
  {
    BSNode * topnode;
    void * nodedata;

    stack->used--;
    if (stack->used > 0)
      stack->top--;
    if (stack->top == -1)
      stack->top = stack->size - 1;

    topnode = stack->nodes + (stack->top * sizeof(BSNode));
    nodedata = topnode->data;

    return nodedata;
  }
}

/* Frees resources used by the stack */

void BSDestroy(BoundedStack * stack)
{
  /*
  int i = 0;
  for (i = 0; i < stack->used; i++)
  {
    BSNode * thisnode = stack->nodes + (i * sizeof(BSNode));
    free(thisnode->data);
  }
  */
  free(stack->nodes);
  free(stack);
}

/** Private Functions */

/* Shows values of used nodes in the stack (assumed to be integers) */

void BSShow(BoundedStack * stack)
{
  int i = 0;
  int * value;
  for (i = 0; i < stack->used; i++)
  {
    BSNode * thisnode = stack->nodes + (i * sizeof(BSNode));
    value = (int *) thisnode->data;
  }
}

/* Demonstrates use of the BoundedStack ADT */

void testFunction(int argc, char ** argv)
{
  BoundedStack * stack = BSCreate(5);
  int * popvalue = NULL;

  int * node1 = (int *) malloc(sizeof(int));
  int * node2 = (int *) malloc(sizeof(int));
  int * node3 = (int *) malloc(sizeof(int));
  int * node4 = (int *) malloc(sizeof(int));
  int * node5 = (int *) malloc(sizeof(int));
  int * node6 = (int *) malloc(sizeof(int));
  int * node7 = (int *) malloc(sizeof(int));

  *node1 = 5;
  *node2 = 6;
  *node3 = 7;
  *node4 = 8;
  *node5 = 9;
  *node6 = 10;
  *node7 = 11;

  BSPush(stack, node1);
  BSPush(stack, node2);
  BSPush(stack, node3);
  BSPush(stack, node4);
  BSPush(stack, node5);
  BSPush(stack, node6);

  BSShow(stack);

  popvalue = (int *) BSPop(stack);
  printf("pop value = %d\n", *popvalue);

  BSDestroy(stack);
}
