#include <SDL/SDL.h>
#include <stdlib.h>
#include <string.h>
#include "bstack.h"
#include "history.h"

unsigned int redoleft = 0; /* how many times user can redo action */

void historySave(SDL_Surface * image, BoundedStack * history)
{
  const int width = image->w;
  const int height = image->h;
  const int bpp = image->format->BitsPerPixel;
  const int totalpixelbytes = width * height * bpp / 8;

  HistoryEntry * historyentry = (HistoryEntry *) malloc(sizeof(HistoryEntry));
  historyentry->width = image->w;
  historyentry->height = image->h;
  historyentry->pixeldata = (Uint32 *) malloc(totalpixelbytes);

  if (SDL_MUSTLOCK(image))
    SDL_LockSurface(image);

  memcpy((char *) historyentry->pixeldata, (char *) image->pixels,
      totalpixelbytes);

  if (SDL_MUSTLOCK(image))
    SDL_UnlockSurface(image);

  redoleft = 0;

  BSPush(history, historyentry);
}

void historyUndo(SDL_Surface * image, BoundedStack * history)
{
  if (history->used > 1)
  {
    HistoryEntry * previous = (HistoryEntry *) BSPop(history);

    const int bpp = 32;
    const int width = previous->width;
    const int height = previous->height;
    const int totalpixelbytes = width * height * bpp / 8;
    Uint32 * pixelbuffer = previous->pixeldata;

    redoleft++;

    image = SDL_SetVideoMode(width, height, bpp, SDL_SWSURFACE);

    if (SDL_MUSTLOCK(image))
      SDL_LockSurface(image);

    memcpy((char *) image->pixels, (char *) pixelbuffer, totalpixelbytes);

    if (SDL_MUSTLOCK(image))
      SDL_UnlockSurface(image);

    SDL_Flip(image);
  }
}

void historyRedo(SDL_Surface * image, BoundedStack * history)
{
  BSNode * currentnode = NULL;
  HistoryEntry * currenthistent = NULL;

  history->top = (history->top + 1) % history->size;

  currentnode = (BSNode *) history->nodes
      + (history->top * sizeof(BSNode));
  currenthistent = (HistoryEntry *) currentnode->data;

  if (currentnode->data != NULL && redoleft > 0)
  {
    const int bpp = 32;
    const int width = currenthistent->width;
    const int height = currenthistent->height;
    const int totalpixelbytes = width * height * bpp / 8;
    Uint32 * pixelbuffer = currenthistent->pixeldata;

    image = SDL_SetVideoMode(width, height, bpp, SDL_SWSURFACE);

    if (SDL_MUSTLOCK(image))
      SDL_LockSurface(image);

    memcpy((char *) image->pixels, (char *) pixelbuffer, totalpixelbytes);

    if (SDL_MUSTLOCK(image))
      SDL_UnlockSurface(image);

    SDL_Flip(image);

    if (history->used < history->size)
      history->used++;

    redoleft--;
  }
  else
  {
    if (history->used > 0)
      history->top--;
    if (history->top == -1)
      history->top = history->size - 1;
  }
}

